<x-meta name="title" content="the title"/>
<x-meta name="subtitle" content="the subtitle"/>
<x-meta foo="foo" bar="bar" baz="baz"/>
<x-meta subtitle="new subtitle" baz="BAZ"/>
<title data-render="meta.title"></title>
